#!/usr/bin/python
# -*- coding: utf-8 -*-
from hvac.exceptions import InvalidPath

from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_delete
version_added: "3.4.0"
short_description: Hashicorp Vault delete module
description:
    - Module to delete from Hashicorp Vault. Consider using `hashivault_secret` instead.
options:
    version:
        description:
            - version of the kv engine (int)
        default: 1
    mount_point:
        description:
            - secret mount point
        default: secret
    secret:
        description:
            - secret to delete.
    permanent:
        description:
            - delete all versions and metadata for a given secret (only effective for kv engine version 2).
        default: false
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_delete:
        secret: giant
'''


def main():
    argspec = hashivault_argspec()
    argspec['version'] = dict(required=False, type='int', default=1)
    argspec['mount_point'] = dict(required=False, type='str', default='secret')
    argspec['secret'] = dict(required=True, type='str')
    argspec['permanent'] = dict(required=False, type='bool', default=False)
    module = hashivault_init(argspec)
    result = hashivault_delete(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_delete(params):
    result = {"changed": False, "rc": 0}
    client = hashivault_auth_client(params)
    version = params.get('version')
    mount_point = params.get('mount_point')
    secret = params.get('secret')
    permanent = params.get('permanent')
    if secret.startswith('/'):
        secret = secret.lstrip('/')
        mount_point = ''
    if mount_point:
        secret_path = '%s/%s' % (mount_point, secret)
    else:
        secret_path = secret

    try:
        if version == 2:
            if permanent:
                returned_data = client.secrets.kv.v2.delete_metadata_and_all_versions(secret, mount_point=mount_point)
            else:
                returned_data = client.secrets.kv.v2.delete_latest_version_of_secret(secret, mount_point=mount_point)
        else:
            returned_data = client.secrets.kv.v1.delete_secret(secret, mount_point=mount_point)
    except InvalidPath:
        result['msg'] = u"Secret %s nonexistent" % secret_path
        result['changed'] = False
        return result
    except Exception as e:
        result['rc'] = 1
        result['failed'] = True
        error_string = "%s(%s)" % (e.__class__.__name__, e)
        result['msg'] = u"Error %s deleting %s" % (error_string, secret_path)
        return result
    if returned_data:
        result['data'] = returned_data.text
    result['msg'] = u"Secret %s deleted" % secret_path
    result['changed'] = True
    return result


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper
from hvac.exceptions import InvalidPath

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_radius_config
version_added: "4.7.0"
short_description: Hashicorp Vault RADIUS configuration module
description:
    - Module to configure the RADIUS authentication method in Hashicorp Vault.
options:
    mount_point:
        description:
            - location where this auth_method is mounted. also known as "path"
        default: radius
    host:
        description:
            - The RADIUS server to connect to. Examples: radius.myorg.com, 127.0.0.1
        required: True
    secret:
        description:
            - The RADIUS shared secret
        required: True
    port:
        description:
            - The UDP port where the RADIUS server is listening on. Defaults is 1812.
        default: 1812
    unregistered_user_policies:
        description:
            - A comma-separated list of policies to be granted to unregistered users.
        default: []
    dial_timeout:
        description:
            - Number of seconds to wait for a backend connection before timing out. Default is 10.
        default: 10
    nas_port:
        description:
            - The NAS-Port attribute of the RADIUS request. Defaults is 10.
        default: 10
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_radius_config:
        host: "radius.myorg.com"
        secret: "my_radius_secret"
        port: 1812
        dial_timeout: 10
        nas_port: 10
        unregistered_user_policies:
          - default
- hosts: localhost
  tasks:
    - hashivault_radius_config:
        host: "127.0.0.1"
        secret: "{{ radius_secret }}"
        mount_point: "radius"
'''


def main():
    argspec = hashivault_argspec()
    argspec['mount_point'] = dict(required=False, type='str', default='radius')
    argspec['host'] = dict(required=True, type='str')
    argspec['secret'] = dict(required=True, type='str', no_log=True)
    argspec['port'] = dict(required=False, type='int', default=1812)
    argspec['unregistered_user_policies'] = dict(required=False, type='list', default=[])
    argspec['dial_timeout'] = dict(required=False, type='int', default=10)
    argspec['nas_port'] = dict(required=False, type='int', default=10)

    module = hashivault_init(argspec, supports_check_mode=True)
    result = hashivault_radius_config(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_radius_config(module):
    params = module.params
    client = hashivault_auth_client(params)
    changed = False
    desired_state = dict()
    desired_state['mount_point'] = params.get('mount_point')
    desired_state['host'] = params.get('host')
    desired_state['secret'] = params.get('secret')
    desired_state['port'] = params.get('port')
    desired_state['unregistered_user_policies'] = params.get('unregistered_user_policies')
    desired_state['dial_timeout'] = params.get('dial_timeout')
    desired_state['nas_port'] = params.get('nas_port')

    # if secret is None, remove it from desired state since we can't compare
    if desired_state['secret'] is None:
        del desired_state['secret']

    # check current config
    current_state = dict()
    try:
        result = client.auth.radius.read_configuration(
            mount_point=desired_state['mount_point'])['data']
        # map keys from Vault response to desired state keys
        current_state['host'] = result.get('host', '')
        current_state['port'] = result.get('port', 1812)
        current_state['unregistered_user_policies'] = result.get('unregistered_user_policies', [])
        current_state['dial_timeout'] = result.get('dial_timeout', 10)
        current_state['nas_port'] = result.get('nas_port', 10)
    except InvalidPath:
        pass

    # If no current config exists, we need to configure
    if not current_state:
        changed = True
    else:
        # check if current config matches desired config values, if they match, set changed to false to prevent action
        for k, v in current_state.items():
            if k in desired_state and v != desired_state[k]:
                changed = True
                break

    # if configs dont match and checkmode is off, complete the change
    if changed and not module.check_mode:
        client.auth.radius.configure(**desired_state)

    return {'changed': changed}


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_radius_user
version_added: "4.7.0"
short_description: Hashicorp Vault RADIUS user management module
description:
    - Module to manage RADIUS users in Hashicorp Vault.
options:
    name:
        description:
            - username to create/update/delete
        required: True
    policies:
        description:
            - List of policies associated with the user
        default: []
    state:
        description:
            - whether to create/update or delete the user
        choices: ['present', 'absent']
        default: present
    mount_point:
        description:
            - The "path" the auth backend is mounted on
        default: radius
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_radius_user:
        name: 'bob'
        policies:
          - 'default'
          - 'bob-policy'
- hosts: localhost
  tasks:
    - hashivault_radius_user:
        name: 'alice'
        policies:
          - 'alice-policy'
        state: present
- hosts: localhost
  tasks:
    - hashivault_radius_user:
        name: 'old-user'
        state: absent
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['policies'] = dict(required=False, type='list', default=[])
    argspec['state'] = dict(required=False, choices=['present', 'absent'], default='present')
    argspec['mount_point'] = dict(required=False, type='str', default='radius')
    module = hashivault_init(argspec)
    result = hashivault_radius_user(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


def hashivault_radius_user_update(user_details, client, user_name, user_policies, mount_point):
    changed = False

    # existing policies
    if user_details['policies'] is not None:
        if set(user_details['policies']) != set(user_policies):
            changed = True
    # new policies and none existing
    elif len(user_policies) > 0:
        changed = True

    if changed:
        try:
            response = client.auth.radius.register_user(
                username=user_name,
                policies=user_policies,
                mount_point=mount_point
            )
        except Exception as e:
            return {'failed': True, 'msg': str(e)}
        if response.status_code == 204:
            return {'changed': True}
        return {'changed': True, 'data': response}
    return {'changed': False}


def hashivault_radius_user_create_or_update(params):
    client = hashivault_auth_client(params)
    user_name = params.get('name')
    mount_point = params.get('mount_point')
    user_policies = params.get('policies')
    try:
        user_details = client.auth.radius.read_user(username=user_name, mount_point=mount_point)
    except Exception:
        client.auth.radius.register_user(
            username=user_name,
            policies=user_policies,
            mount_point=mount_point
        )
        return {'changed': True}
    return hashivault_radius_user_update(user_details['data'], client, user_name=user_name,
                                         user_policies=user_policies,
                                         mount_point=mount_point)


def hashivault_radius_user_delete(params):
    client = hashivault_auth_client(params)
    user_name = params.get('name')
    mount_point = params.get('mount_point')

    try:
        client.auth.radius.read_user(username=user_name, mount_point=mount_point)
    except Exception:
        return {'changed': False}
    client.auth.radius.delete_user(username=user_name, mount_point=mount_point)
    return {'changed': True}


@hashiwrapper
def hashivault_radius_user(params):
    state = params.get('state')
    if state == 'present':
        return hashivault_radius_user_create_or_update(params)
    elif state == 'absent':
        return hashivault_radius_user_delete(params)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
from hvac.exceptions import InvalidPath

from ansible.module_utils.hashivault import is_state_changed
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_write
version_added: "0.1"
short_description: Hashicorp Vault write module
description:
    - Module to write to Hashicorp Vault. Consider using `hashivault_secret` instead.
options:
    version:
        description:
            - version of the kv engine (int)
        default: 1
    mount_point:
        description:
            - secret mount point
        default: secret
    secret:
        description:
            - vault secret to write.
    data:
        description:
            - Keys and values to write.
    alternate_data:
        description:
            - Keys and values to write. Use this if you need the returned data.
    cas:
        description:
            - Check and set curent version v2 only
    update:
        description:
            - This option is deprecated. Update the secret rather than overwrite. The module will read the secret and
              overlay with the data provided and write.
        default: False
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_write:
        secret: giant
        data:
            foo: foe
            fie: fum
'''


def main():
    argspec = hashivault_argspec()
    argspec['version'] = dict(required=False, type='int', default=1)
    argspec['mount_point'] = dict(required=False, type='str', default='secret')
    argspec['secret'] = dict(required=True, type='str')
    argspec['update'] = dict(required=False, default=False, type='bool')
    argspec['data'] = dict(required=False, default={}, type='dict', no_log=True)
    argspec['alternate_data'] = dict(required=False, default={}, type='dict')
    argspec['cas'] = dict(required=False, type='int')
    module = hashivault_init(argspec, supports_check_mode=True)
    result = hashivault_write(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_write(module):
    result = {"changed": False, "rc": 0}
    params = module.params
    client = hashivault_auth_client(params)
    version = params.get('version')
    mount_point = params.get('mount_point')
    secret = params.get('secret')
    data = params.get('data')
    data = data or params.get('alternate_data')
    cas = params.get('cas')

    if secret.startswith('/'):
        secret = secret.lstrip('/')
        mount_point = ''
    if mount_point:
        secret_path = '%s/%s' % (mount_point, secret)
    else:
        secret_path = secret

    changed = True
    write_data = data

    if params.get('update') or module.check_mode:
        # Do not move these reads outside of the update
        read_data = None
        try:
            if version == 2:
                read_data = client.secrets.kv.v2.read_secret_version(secret, mount_point=mount_point)
            else:
                read_data = client.secrets.kv.v1.read_secret(secret, mount_point=mount_point)
        except InvalidPath:
            read_data = None
        except Exception as e:
            result['rc'] = 1
            result['failed'] = True
            error_string = "%s(%s)" % (e.__class__.__name__, e)
            result['msg'] = u"Error %s reading %s" % (error_string, secret_path)
            return result
        if not read_data:
            read_data = {}
        read_data = read_data.get('data', {})

        if version == 2:
            read_data = read_data.get('data', {})

        write_data = dict(read_data)
        write_data.update(data)

        # result['write_data'] = write_data
        # result['read_data'] = read_data
        changed = is_state_changed(write_data, read_data)

    if changed:
        if not module.check_mode:
            try:
                if version == 2:
                    returned_data = client.secrets.kv.v2.create_or_update_secret(mount_point=mount_point, cas=cas,
                                                                                 path=secret, secret=write_data)
                else:
                    returned_data = client.write_data(secret_path, data=write_data)
                if returned_data:
                    from requests.models import Response
                    if isinstance(returned_data, Response):
                        result['data'] = returned_data.text
                    else:
                        result['data'] = returned_data
            except Exception as e:
                result['rc'] = 1
                result['failed'] = True
                error_string = "%s(%s)" % (e.__class__.__name__, e)
                result['msg'] = u"Error %s writing %s" % (error_string, secret_path)
                return result

        result['msg'] = u"Secret %s written" % secret_path
    result['changed'] = changed
    return result


if __name__ == '__main__':
    main()
